#This source code is able to freely use for Blender and users.

bl_info = {
    "name": "Add point of orthogonal",
    "author": "TMO",
    "version": (0, 1, 0),
    "blender": (2, 6, 0),
    "location": "View3D > Add > Mesh > Add point of orthogonal",
    "description": "Add point of orthogonal",
    "warning": "",
    "wiki_url": "http://www.sousakuba.com/blender/index.html",
    "tracker_url": "",
    "category": "3D View"}

import bpy
from bpy.props import *
from bpy_extras import object_utils
import mathutils

class MSG():
    msg_type = ''
    msg = ''

class MB(bpy.types.Operator, MSG):
    bl_idname = 'va.msg_box'
    bl_label = ''

    def draw(self, context):
        if self.msg_type == 'ERROR':
            self.t = 'ERROR'
        else:
            self.t = 'NONE'

        layout = self.layout
        row = layout.split(0.25)
        row.label(self.msg_type, icon = self.t)
        row.label(self.msg)

    def execute(self, context):
        return {'FINISHED'}

    def invoke(self, context, event):
        return context.window_manager.invoke_popup(self)

def vlen(v):
	return pow( v.x*v.x + v.y*v.y + v.z*v.z, 0.5 )

def normal_vector(v):
    v_len = vlen(v);
    v.x /= v_len;
    v.y /= v_len;
    v.z /= v_len;

def zoom(v, val):
    v.x *= val;
    v.y *= val;
    v.z *= val;

def cross_product(l,r):
    return mathutils.Vector( (l.y * r.z) - (l.z * r.y), (l.z * r.x) - (l.x * r.z), (l.x * r.y) - (l.y * r.x) )

def dot_product(l,r):
    return l.x * r.x + l.y * r.y + l.z * r.z

class main(bpy.types.Operator):
    bl_idname="mesh.add_point_of_orthogonal"
    bl_label="Add point of orthogonal"
    bl_options = {'REGISTER', 'UNDO'}

    def draw(self, context):
        return

    def execute(self, context):

        #need for update select
        bpy.ops.object.editmode_toggle()
        bpy.ops.object.editmode_toggle()

        mesh = bpy.context.active_object.data

        if mesh.total_vert_sel != 3 or mesh.total_edge_sel != 1:
            MSG.msg_type = 'ERROR'
            MSG.msg = 'Please select 1 vert and 1 edge'
            bpy.ops.va.msg_box('INVOKE_DEFAULT')
            return {'FINISHED'}

        v_sel = [e for e in mesh.vertices if e.select]
        e_sel = [e for e in mesh.edges if e.select]
        edge = e_sel[0]

        A = mesh.vertices[ edge.vertices[0] ];
        B = mesh.vertices[ edge.vertices[1] ];
        P = v_sel[0];

        if v_sel[0].index != edge.vertices[0] and v_sel[0].index != edge.vertices[1]:
            P =  v_sel[0];
        elif v_sel[1].index != edge.vertices[0] and v_sel[1].index != edge.vertices[1]:
            P =  v_sel[1];
        else:
            P =  v_sel[2];

        #print("A:%f %f %f\n", A.co.x, A.co.y, A.co.z)
        #print("B:%f %f %f\n", B.co.x, B.co.y, B.co.z)
        #print("P:%f %f %f\n", P.co.x, P.co.y, P.co.z)

        vAP = mathutils.Vector( ( P.co.x - A.co.x, P.co.y - A.co.y, P.co.z - A.co.z ) )
        normalAB = mathutils.Vector( ( B.co.x - A.co.x, B.co.y - A.co.y, B.co.z - A.co.z ) )
        normal_vector( normalAB )

        d = dot_product( normalAB, vAP )

        zoom(normalAB, d)

	#----------------------------------------------------
        cursor = bpy.context.scene.cursor_location

        verts = []
        verts.append( [ A.co.x + normalAB.x-cursor[0], A.co.y + normalAB.y-cursor[1], A.co.z + normalAB.z-cursor[2] ] )
        #verts.append( [ 1,1,1 ] )

        n_mesh = bpy.data.meshes.new("mesh")
        n_mesh.from_pydata(verts, [], [])
        n_mesh.update()
        object_utils.object_data_add(context, n_mesh, operator=None)



        return {'FINISHED'}

def menu_func(self, context):
    self.layout.operator(main.bl_idname, main.bl_label, icon="PLUGIN")

def register():
    bpy.utils.register_module(__name__)

    bpy.types.INFO_MT_mesh_add.append(menu_func)

def unregister():
    bpy.utils.unregister_module(__name__)

    bpy.types.INFO_MT_mesh_add.remove(menu_func)

if __name__ == "__main__":
    register()
